import com.nerius.math.geom.*;
import com.nerius.math.xform.*;

public final class CorkscrewGenerator extends GenerationUtils {

  public static void main(String[] args) {
    // NOTE TO MYSELF: Don't support offset.  It's too complicated.
    final int outerRadius = Integer.parseInt(args[0]);
    final int innerRadius = Integer.parseInt(args[1]);
    final int altitude = Integer.parseInt(args[2]);
    final int lip = Integer.parseInt(args[3]);
    final int slices = Integer.parseInt(args[4]);
    final int outerThickness = Integer.parseInt(args[5]);
    final int innerThickness = Integer.parseInt(args[6]);
    final boolean adjustForOverlappingCylinderCuts = Boolean.valueOf(args[7]);
    final Point3D[][][] mesh = new Point3D[slices + 1][2][2];
    {
      final Point3D[] outerCurve = generatePipeCurve
        (outerRadius, slices, 0, 0.0d,
         adjustForOverlappingCylinderCuts ? ADJUST_FOR_OVERLAPPING_PIPE_CUTS :
         ADJUST_NONE);
      final Point3D[] innerCurve = generatePipeCurve
        (innerRadius, slices, 0, 0.0d,
         adjustForOverlappingCylinderCuts ? ADJUST_FOR_OVERLAPPING_PIPE_CUTS :
         ADJUST_NONE);
      final AxisRotation3D rot = new AxisRotation3D
        (AxisRotation3D.X_AXIS, Math.PI / 2);
      for (int i = 0; i <= slices; i++) {
        outerCurve[i] = rot.transform(outerCurve[i]);
        innerCurve[i] = rot.transform(innerCurve[i]);        
      }
      final double stepsPerSlice = ((double) altitude) / slices;
      for (int i = 0; i <= slices; i++) {
        mesh[i][0][0] = new Point3D(outerCurve[i].x,
                                    outerCurve[i].y,
                                    i * stepsPerSlice);
        mesh[i][0][1] = new Point3D(outerCurve[i].x,
                                    outerCurve[i].y,
                                    i * stepsPerSlice - outerThickness);
        mesh[i][1][0] = new Point3D(innerCurve[i].x,
                                    innerCurve[i].y,
                                    i * stepsPerSlice - lip);
        mesh[i][1][1] = new Point3D(innerCurve[i].x,
                                    innerCurve[i].y,
                                    i * stepsPerSlice -
                                    (innerThickness + lip));
      }
    }
    startMap();
    for (int i = 0; i < slices; i++) {
      startBrush();
      writeFace(mesh[i][0][0],
                mesh[i][1][0],
                mesh[i + 1][0][0]);
      if (outerThickness != 0) {
        writeFace(mesh[i][0][0],
                  mesh[i][0][1],
                  mesh[i][1][1]);
      }
      else {
        writeFace(mesh[i][0][0],
                  mesh[i][1][1],
                  mesh[i][1][0]);
      }
      if (outerThickness != 0) {
        writeFace(mesh[i][0][0],
                  mesh[i + 1][0][0],
                  mesh[i + 1][0][1]);
      }
      if (innerThickness != 0) {
        writeFace(mesh[i + 1][0][0],
                  mesh[i][1][0],
                  mesh[i][1][1]);
      }
      else {
        writeFace(mesh[i + 1][0][0],
                  mesh[i][1][0],
                  mesh[i + 1][0][1]);
      }
      writeFace(mesh[i][0][1],
                mesh[i + 1][0][1],
                mesh[i][1][1]);
      endBrush();
      startBrush();
      writeFace(mesh[i + 1][0][0],
                mesh[i][1][0],
                mesh[i + 1][1][0]);
      if (outerThickness != 0) {
        writeFace(mesh[i + 1][0][0],
                  mesh[i + 1][0][1],
                  mesh[i][1][1]);
      }
      else {
        writeFace(mesh[i + 1][0][0],
                  mesh[i][1][1],
                  mesh[i][1][0]);
      }
      if (innerThickness != 0) {
        writeFace(mesh[i + 1][0][0],
                  mesh[i + 1][1][0],
                  mesh[i + 1][1][1]);
      }
      else {
        writeFace(mesh[i + 1][0][0],
                  mesh[i + 1][1][0],
                  mesh[i + 1][0][1]);
      }
      if (innerThickness != 0) {
        writeFace(mesh[i][1][0],
                  mesh[i][1][1],
                  mesh[i + 1][1][1]);
      }
      writeFace(mesh[i + 1][0][1],
                mesh[i + 1][1][1],
                mesh[i][1][1]);
      endBrush();
    }
    endMap();
  }

}
