import com.nerius.math.geom.*;
import com.nerius.math.xform.*;

/**
 * Generates a quarter of an upright cylinder.
 */
public final class CylinderGenerator extends GenerationUtils {

  public static void main(String[] args) {
    final int outerRadius = Integer.parseInt(args[0]);
    final int innerRadius = Integer.parseInt(args[1]);
    final int slices = Integer.parseInt(args[2]);
    final int height = Integer.parseInt(args[3]);
    final boolean overlappingBrushes = Boolean.valueOf(args[4]);
    final boolean adjustForOverlappingCylinderCuts = Boolean.valueOf(args[5]);
    final Point3D[] outerCurve = generatePipeCurve
      (outerRadius, slices, 0, 0.0d,
       adjustForOverlappingCylinderCuts ? ADJUST_FOR_OVERLAPPING_PIPE_CUTS :
       ADJUST_NONE);
    final Point3D[] innerCurve = generatePipeCurve
      (innerRadius, slices, 0, 0.0d,
       (adjustForOverlappingCylinderCuts ? ADJUST_FOR_OVERLAPPING_PIPE_CUTS :
        ADJUST_NONE));
    final AxisRotation3D rot = new AxisRotation3D
      (AxisRotation3D.X_AXIS, Math.PI / 2);
    for (int i = 0; i <= slices; i++) {
      outerCurve[i] = rot.transform(outerCurve[i]);
      innerCurve[i] = rot.transform(innerCurve[i]);
    }
    startMap();
    for (int i = 0; i < slices; i++) {
      // When innerRadius approaches 0, this code will generate
      // degenerate brushes because consecutive inner points (rounded to
      // nearest integer) may be equal.
      if (innerRadius == 0.0d) {
        writeBrushExtendZ(-height,
                          outerCurve[i],
                          innerCurve[i],
                          outerCurve[i + 1],
                          false);
      }
      else {
        final Point3D pt1, pt2, pt3, pt4;
        pt1 = outerCurve[i];
        pt4 = outerCurve[i + 1];
        if (!overlappingBrushes) {
          pt2 = innerCurve[i];
          pt3 = innerCurve[i + 1];
        }
        else {
          final double pt2x = Math.round(innerCurve[i].x);
          final double pt2y = Math.round(innerCurve[i].y);
          final double pt3x = Math.round(innerCurve[i + 1].x);
          final double pt3y = Math.round(innerCurve[i + 1].y);
          pt2 = new Point3D(pt2x + (pt2x - pt3x), pt2y + (pt2y - pt3y), 0);
          pt3 = new Point3D(pt3x + (pt3x - pt2x), pt3y + (pt3y - pt2y), 0);
        }
        writeBrushExtendZ(-height, pt1, pt2, pt3, pt4);
      }
    }
    endMap();
  }

}
