import com.nerius.math.geom.*;
import com.nerius.math.xform.*;

/**
 * Generates half of a disc.
 */
public final class DiscGenerator extends GenerationUtils {

  public static void main(String[] args) {
    final int radius = Integer.parseInt(args[0]);
    final int faces = Integer.parseInt(args[1]);
    final int height = Integer.parseInt(args[2]);
    final boolean adjustForOverlappingCylinderCuts = Boolean.valueOf(args[3]);
    final Point3D[] curve;
    if (adjustForOverlappingCylinderCuts) {
      if ((faces % 2) != 0) {
        System.err.println("WARNING: Cannot adjust for overlapping " +
                           "cylinder cuts when number of faces\n" +
                           "in half-disc is odd.");
        curve = generateCurveNormally(radius, faces);
      }
      else {
        final Point3D[] curveTemp = generatePipeCurve
          (radius, faces / 2, 0, 0.0d, ADJUST_FOR_OVERLAPPING_PIPE_CUTS);
        curve = new Point3D[faces + 1];
        AxisRotation3D rot = new AxisRotation3D
          (AxisRotation3D.X_AXIS, Math.PI / 2);
        for (int i = 0; i < curveTemp.length; i++) {
          curve[i] = rot.transform(curveTemp[i]);
        }
        rot = new AxisRotation3D(AxisRotation3D.Z_AXIS, Math.PI / 2);
        for (int i = curveTemp.length; i <= faces; i++) {
          curve[i] = rot.transform(curve[i - curveTemp.length + 1]);
        }
      }
    }
    else {
      curve = generateCurveNormally(radius, faces);
    }
    startMap();
    for (int i = 0; i < faces / 2; i++) {
      if (i + 1 == faces / 2) {
        if ((faces % 2) == 1) {
          writeBrushExtendZ(-height,
                            curve[i],
                            curve[i + 3],
                            curve[i + 2],
                            curve[i + 1]);
        }
        else {
          writeBrushExtendZ(-height,
                            curve[i],
                            curve[i + 2],
                            curve[i + 1],
                            false);
        }
      }
      else {
        final int vert = faces - i;
        writeBrushExtendZ(-height,
                          curve[i],
                          curve[vert],
                          curve[vert - 1],
                          curve[i + 1]);
      }
    }
    endMap();
  }

  private static Point3D[] generateCurveNormally(final int radius,
                                                 final int faces) {
    final Point3D pt = new Point3D(radius, 0, 0);
    final Point3D[] curve = new Point3D[faces + 1];
    for (int i = 0; i <= faces; i++) {
      final AxisRotation3D rot = new AxisRotation3D
        (AxisRotation3D.Z_AXIS, (Math.PI * i) / faces);
      curve[i] = rot.transform(pt);
    }
    return curve;
  }

}
