import com.nerius.math.geom.*;
import com.nerius.math.xform.*;

public final class FunnelGenerator extends GenerationUtils {

  public static void main(String[] args) {
    final int topRadius = Integer.parseInt(args[0]);
    final int bottomRadius = Integer.parseInt(args[1]);
    final int height = Integer.parseInt(args[2]);
    final int slices = Integer.parseInt(args[3]);
    final int crossSection = Integer.parseInt(args[4]);
    final boolean offsetTopInsteadOfBottom = Boolean.valueOf(args[5]);
    final boolean extendDownInsteadOfSquare = Boolean.valueOf(args[6]);
    final boolean overlappingBrushes = Boolean.valueOf(args[7]);
    final boolean generateOnlySeam = Boolean.valueOf(args[8]);
    final boolean adjustSeamForJoining = Boolean.valueOf(args[9]);
    // STEP 1: Generate the top and bottom curves through 3-space.
    // Dependent on topRadius, bottomRadius, height, slices,
    // offsetTopInsteadOfBottom, and adjustSeamForJoining.
    final Point3D[] topCurve;
    final Point3D[] bottomCurve;
    {
      // NOTE TO MYSELF: Handle case of either radius being zero.
      topCurve = new Point3D[slices + 1];
      bottomCurve = new Point3D[slices + 1];
      final Point3D topPt = new Point3D(topRadius, 0, 0);
      final Point3D bottomPt = new Point3D(bottomRadius, 0, -height);
      AxisRotation3D rot = new AxisRotation3D
        (AxisRotation3D.Z_AXIS, Math.PI / (4 * slices));
      if (!offsetTopInsteadOfBottom) {
        topCurve[0] = topPt;
        bottomCurve[0] = rot.transform(bottomPt);
      }
      else {
        topCurve[0] = rot.transform(topPt);
        bottomCurve[0] = bottomPt;
      }
      for (int i = 1; i <= slices; i++) {
        rot = new AxisRotation3D(AxisRotation3D.Z_AXIS,
                                 (Math.PI * i) / (2 * slices));
        topCurve[i] = rot.transform(topCurve[0]);
        bottomCurve[i] = rot.transform(bottomCurve[0]);
      }
      if (adjustSeamForJoining) {
        if (!offsetTopInsteadOfBottom) {
          bottomCurve[slices] = new Point3D(0, bottomRadius, -height);
        }
        else {
          topCurve[slices] = new Point3D(0, topRadius, 0);
        }
      }
    }
    startMap();
    for (int i = 0; i < slices * 2; i++) {
      if (generateOnlySeam && i != slices * 2 - 1) continue;
      if ((i % 2) == 0) { // i is even.
        if (!offsetTopInsteadOfBottom) {
          if (topRadius != 0) {
            writeBrushExtendWrapper
              (crossSection, topCurve[i / 2], bottomCurve[i / 2],
               topCurve[i / 2 + 1], i >= slices, extendDownInsteadOfSquare,
               overlappingBrushes);
          } // else topRadius is 0, don't write brush.
        }
        else { // offsetTopInsteadOfBottom is true.
          if (bottomRadius != 0) {
            writeBrushExtendWrapper
              (crossSection, bottomCurve[i / 2], bottomCurve[i / 2 + 1],
               topCurve[i / 2], i >= slices, extendDownInsteadOfSquare,
               overlappingBrushes);
          } // else bottomRadius is 0, don't write brush.
        }
      }
      else { // i is odd.
        if (!offsetTopInsteadOfBottom) {
          if (bottomRadius != 0) {
            writeBrushExtendWrapper
              (crossSection, topCurve[i / 2 + 1], bottomCurve[i / 2],
               bottomCurve[i / 2 + 1], i >= slices, extendDownInsteadOfSquare,
               overlappingBrushes);
          } // else bottomRadius is 0, don't write brush.
        }
        else { // offsetTopInsteadOfBottom is true.
          if (topRadius != 0) {
            writeBrushExtendWrapper
              (crossSection, topCurve[i / 2], bottomCurve[i / 2 + 1],
               topCurve[i / 2 + 1], i >= slices, extendDownInsteadOfSquare,
               overlappingBrushes);
          } // else topRadius is 0, don't write brush.
        }
      }
      if (i == slices && (!extendDownInsteadOfSquare) &&
          (!overlappingBrushes)) {
        final Point3D tempTopPt, tempBottomPt;
        if ((i % 2) == 0) {
          tempTopPt = topCurve[i / 2];
          tempBottomPt = bottomCurve[i / 2];
        }
        else {
          if (!offsetTopInsteadOfBottom) {
            tempTopPt = topCurve[i / 2 + 1];
            tempBottomPt = bottomCurve[i / 2];
          }
          else {
            tempTopPt = topCurve[i / 2];
            tempBottomPt = bottomCurve[i / 2 + 1];
          }
        }
        writeBrushExtendY(crossSection,
                          tempTopPt,
                          new Point3D(crossSection,
                                      tempTopPt.y,
                                      0),
                          new Point3D(crossSection,
                                      tempBottomPt.y,
                                      -height),
                          tempBottomPt);
      }
    }
    endMap();
  }

  private static void writeBrushExtendWrapper(int crossSection,
                                              Point3D pt1,
                                              Point3D pt2,
                                              Point3D pt3,
                                              boolean pastHalfway,
                                              boolean extendDown,
                                              boolean overlappingBrushes)
  {
    if (!extendDown) {
      if (!pastHalfway) { // Not yet halfway; extend x.
        writeBrushExtendX(crossSection,
                          pt1,
                          pt2,
                          pt3,
                          overlappingBrushes);
      }
      else { // We're past halfway; extend y instead of x.
        writeBrushExtendY(crossSection,
                          pt1,
                          pt2,
                          pt3,
                          overlappingBrushes);
      }
    }
    else { // extendDown is true.
      writeBrushExtendZ(-crossSection,
                        pt1,
                        pt2,
                        pt3,
                        overlappingBrushes);
    }
  }

}
