import com.nerius.math.geom.*;
import com.nerius.math.xform.*;

/**
 * Generates a quarter pipe ramp.  All vertices are rotated
 * one half of a slice down.
 */
// NOTE TO MYSELF: We don't need shingles for this.
public final class OffsetPipeGenerator extends GenerationUtils {

  public static void main(String[] args) {
    final int radius = Integer.parseInt(args[0]);
    final int slices = Integer.parseInt(args[1]);
    final int crossSection = Integer.parseInt(args[2]);
    final int length = Integer.parseInt(args[3]);
    final boolean overlappingBrushes = Boolean.valueOf(args[4]);
    final boolean generateOnlySeam = Boolean.valueOf(args[5]);
    final boolean adjustSeamForJoining = Boolean.valueOf(args[6]);
    final Point3D[] curve = new Point3D[slices + 1];
    final Point3D pt = new Point3D(radius, 0, 0);
    AxisRotation3D rot = new AxisRotation3D
      (AxisRotation3D.Y_AXIS, Math.PI / (4 * slices));
    curve[0] = rot.transform(pt);
    for (int i = 1; i <= slices; i++) {
      rot = new AxisRotation3D(AxisRotation3D.Y_AXIS,
                               (Math.PI * i) / (2 * slices));
      curve[i] = rot.transform(curve[0]);
    }
    if (adjustSeamForJoining) {
      curve[slices] = new Point3D(0, 0, -radius);
    }
    startMap();
    for (int i = 0; i < slices; i++) {
      if (generateOnlySeam && i != slices - 1) continue;
      final Point3D pt1, pt2, pt3, pt4;
      if (!overlappingBrushes) {
        pt1 = curve[i];
        pt2 = curve[i + 1];
        pt3 = new Point3D(pt2.x, length, pt2.z);
        pt4 = new Point3D(pt1.x, length, pt1.z);
      }
      else {
        final double pt1x = Math.round(curve[i].x);
        final double pt1z = Math.round(curve[i].z);
        final double pt2x = Math.round(curve[i + 1].x);
        final double pt2z = Math.round(curve[i + 1].z);
        pt1 = new Point3D(pt1x + (pt1x - pt2x), 0, pt1z + (pt1z - pt2z));
        pt2 = new Point3D(pt2x + (pt2x - pt1x), 0, pt2z + (pt2z - pt1z));
        pt3 = new Point3D(pt2.x, length, pt2.z);
        pt4 = new Point3D(pt1.x, length, pt1.z);
      }
      if (i < slices / 2) {
        writeBrushExtendX(crossSection, pt1, pt2, pt3, pt4);
      }
      else {
        writeBrushExtendZ(-crossSection, pt1, pt2, pt3, pt4);
        if (i == slices / 2 && !overlappingBrushes) {
          writeBrushExtendZ(-crossSection,
                            curve[i],
                            new Point3D(curve[i].x, length, curve[i].z),
                            new Point3D(crossSection, length, curve[i].z),
                            new Point3D(crossSection, 0, curve[i].z));
        }
      }
    }
    endMap();
  }

}
