import com.nerius.math.geom.*;
import com.nerius.math.xform.*;

/**
 * Generates a quarter pipe ramp.
 */
public final class PipeGenerator extends GenerationUtils {

  public static void main(String[] args) {
    final int radius = Integer.parseInt(args[0]);
    final int slices = Integer.parseInt(args[1]);
    final int crossSection = Integer.parseInt(args[2]);
    final int length = Integer.parseInt(args[3]);
    final boolean overlappingBrushes = Boolean.valueOf(args[4]);
    final int extrudeTopNSlices = Integer.parseInt(args[5]);
    final double extrudeRadiusFactor = Double.parseDouble(args[6]);
    final boolean adjustForOverlappingPipeCuts = Boolean.valueOf(args[7]);
    final boolean adjustForOverlappingBowlCuts = Boolean.valueOf(args[8]);
    int adjustMask = 0;
    if (adjustForOverlappingBowlCuts) {
      adjustMask |= ADJUST_FOR_OVERLAPPING_BOWL_CUTS;
    }
    if (adjustForOverlappingPipeCuts) {
      adjustMask |= ADJUST_FOR_OVERLAPPING_PIPE_CUTS;
    }
    final Point3D[] curve = generatePipeCurve(radius,
                                              slices,
                                              extrudeTopNSlices,
                                              extrudeRadiusFactor,
                                              adjustMask);
    startMap();
    for (int i = 0; i < slices; i++) {
      final Point3D pt1, pt2, pt3, pt4;
      if (!overlappingBrushes) {
        pt1 = curve[i];
        pt2 = curve[i + 1];
        pt3 = new Point3D(pt2.x, length, pt2.z);
        pt4 = new Point3D(pt1.x, length, pt1.z);
      }
      else {
        final double pt1x = Math.round(curve[i].x);
        final double pt1z = Math.round(curve[i].z);
        final double pt2x = Math.round(curve[i + 1].x);
        final double pt2z = Math.round(curve[i + 1].z);
        pt1 = new Point3D(pt1x + (pt1x - pt2x), 0, pt1z + (pt1z - pt2z));
        pt2 = new Point3D(pt2x + (pt2x - pt1x), 0, pt2z + (pt2z - pt1z));
        pt3 = new Point3D(pt2.x, length, pt2.z);
        pt4 = new Point3D(pt1.x, length, pt1.z);
      }
      if (i < (slices + 1) / 2) {
        writeBrushExtendX(crossSection, pt1, pt2, pt3, pt4);
      }
      else {
        writeBrushExtendZ(-crossSection, pt1, pt2, pt3, pt4);
        if (i == (slices + 1) / 2 && !overlappingBrushes) {
          writeBrushExtendZ(-crossSection,
                            curve[i],
                            new Point3D(curve[i].x, length, curve[i].z),
                            new Point3D(crossSection, length, curve[i].z),
                            new Point3D(crossSection, 0, curve[i].z));
        }
      }
    }
    endMap();
  }

}
